typedef struct
{                                       /* opcode structure */
 UBYTE opcode;				/* 8-bit opcode value */
 UBYTE numoperands;
 char name[6];				/* opcode name */
 UBYTE mode;				/* addressing mode */
 UBYTE numcycles;			/* number of cycles - not used */
} opcodeinfo;

/* 6809 ADDRESSING MODES */
#define INH 0
#define DIR 1
#define IND 2
#define REL 3
#define EXT 4
#define IMM 5
#define LREL 6
#define PG2 7                                    /* PAGE SWITCHES - Page 2 */
#define PG3 8                                                    /* Page 3 */

/* number of opcodes in each page */
#define NUMPG1OPS 223
#define NUMPG2OPS 38
#define NUMPG3OPS 9

int numops[3]=
{
 NUMPG1OPS,NUMPG2OPS,NUMPG3OPS,
};

char modenames[9][14]=
{
 "inherent",
 "direct",
 "indexed",
 "relative",
 "extended",
 "immediate",
 "long relative",
 "page 2",
 "page 3",
};

opcodeinfo pg1opcodes[NUMPG1OPS]=
{                           /* page 1 ops */
 0,1,"neg",DIR,6,
 3,1,"com",DIR,6,
 4,1,"lsr",DIR,6,
 6,1,"ror",DIR,6,
 7,1,"asr",DIR,6,
 8,1,"asl",DIR,6,
 9,1,"rol",DIR,6,
 10,1,"dec",DIR,6,
 12,1,"inc",DIR,6,
 13,1,"tst",DIR,6,
 14,1,"jmp",DIR,3,
 15,1,"clr",DIR,6,

 16,1,"page2",PG2,0,
 17,1,"page3",PG3,0,
 18,0,"nop",INH,2,
 19,0,"sync",INH,4,
 22,2,"lbra",LREL,5,
 23,2,"lbsr",LREL,9,
 25,0,"daa",INH,2,
 26,1,"orcc",IMM,3,
 28,1,"andcc",IMM,3,
 29,0,"sex",INH,2,
 30,1,"exg",IMM,8,
 31,1,"tfr",IMM,6,

 32,1,"bra",REL,3,
 33,1,"brn",REL,3,
 34,1,"bhi",REL,3,
 35,1,"bls",REL,3,
 36,1,"bcc",REL,3,
 37,1,"bcs",REL,3,
 38,1,"bne",REL,3,
 39,1,"beq",REL,3,
 40,1,"bvc",REL,3,
 41,1,"bvs",REL,3,
 42,1,"bpl",REL,3,
 43,1,"bmi",REL,3,
 44,1,"bge",REL,3,
 45,1,"blt",REL,3,
 46,1,"bgt",REL,3,
 47,1,"ble",REL,3,

 48,1,"leax",IND,2,
 49,1,"leay",IND,2,
 50,1,"leas",IND,2,
 51,1,"leau",IND,2,
 52,1,"pshs",INH,5,
 53,1,"puls",INH,5,
 54,1,"pshu",INH,5,
 55,1,"pulu",INH,5,
 57,0,"rts",INH,5,
 58,0,"abx",INH,3,
 59,0,"rti",INH,6,
 60,1,"cwai",IMM,20,
 61,0,"mul",INH,11,
 63,0,"swi",INH,19,

 64,0,"nega",INH,2,
 67,0,"coma",INH,2,
 68,0,"lsra",INH,2,
 70,0,"rora",INH,2,
 71,0,"asra",INH,2,
 72,0,"asla",INH,2,
 73,0,"rola",INH,2,
 74,0,"deca",INH,2,
 76,0,"inca",INH,2,
 77,0,"tsta",INH,2,
 79,0,"clra",INH,2,

 80,0,"negb",INH,2,
 83,0,"comb",INH,2,
 84,0,"lsrb",INH,2,
 86,0,"rorb",INH,2,
 87,0,"asrb",INH,2,
 88,0,"aslb",INH,2,
 89,0,"rolb",INH,2,
 90,0,"decb",INH,2,
 92,0,"incb",INH,2,
 93,0,"tstb",INH,2,
 95,0,"clrb",INH,2,

 96,1,"neg",IND,6,
 99,1,"com",IND,6,
 100,1,"lsr",IND,6,
 102,1,"ror",IND,6,
 103,1,"asr",IND,6,
 104,1,"asl",IND,6,
 105,1,"rol",IND,6,
 106,1,"dec",IND,6,
 108,1,"inc",IND,6,
 109,1,"tst",IND,6,
 110,1,"jmp",IND,3,
 111,1,"clr",IND,6,

 112,2,"neg",EXT,7,
 115,2,"com",EXT,7,
 116,2,"lsr",EXT,7,
 118,2,"ror",EXT,7,
 119,2,"asr",EXT,7,
 120,2,"asl",EXT,7,
 121,2,"rol",EXT,7,
 122,2,"dec",EXT,7,
 124,2,"inc",EXT,7,
 125,2,"tst",EXT,7,
 126,2,"jmp",EXT,4,
 127,2,"clr",EXT,7,

 128,1,"suba",IMM,2,
 129,1,"cmpa",IMM,2,
 130,1,"sbca",IMM,2,
 131,2,"subd",IMM,4,
 132,1,"anda",IMM,2,
 133,1,"bita",IMM,2,
 134,1,"lda",IMM,2,
 136,1,"eora",IMM,2,
 137,1,"adca",IMM,2,
 138,1,"ora",IMM,2,
 139,1,"adda",IMM,2,
 140,2,"cmpx",IMM,4,
 141,1,"bsr",REL,7,
 142,2,"ldx",IMM,3,

 144,1,"suba",DIR,4,
 145,1,"cmpa",DIR,4,
 146,1,"sbca",DIR,4,
 147,1,"subd",DIR,6,
 148,1,"anda",DIR,4,
 149,1,"bita",DIR,4,
 150,1,"lda",DIR,4,
 151,1,"sta",DIR,4,
 152,1,"eora",DIR,4,
 153,1,"adca",DIR,4,
 154,1,"ora",DIR,4,
 155,1,"adda",DIR,4,
 156,1,"cpx",DIR,6,
 157,1,"jsr",DIR,7,
 158,1,"ldx",DIR,5,
 159,1,"stx",DIR,5,

 160,1,"suba",IND,4,
 161,1,"cmpa",IND,4,
 162,1,"sbca",IND,4,
 163,1,"subd",IND,6,
 164,1,"anda",IND,4,
 165,1,"bita",IND,4,
 166,1,"lda",IND,4,
 167,1,"sta",IND,4,
 168,1,"eora",IND,4,
 169,1,"adca",IND,4,
 170,1,"ora",IND,4,
 171,1,"adda",IND,4,
 172,1,"cpx",IND,6,
 173,1,"jsr",IND,7,
 174,1,"ldx",IND,5,
 175,1,"stx",IND,5,

 176,2,"suba",EXT,5,
 177,2,"cmpa",EXT,5,
 178,2,"sbca",EXT,5,
 179,2,"subd",EXT,7,
 180,2,"anda",EXT,5,
 181,2,"bita",EXT,5,
 182,2,"lda",EXT,5,
 183,2,"sta",EXT,5,
 184,2,"eora",EXT,5,
 185,2,"adca",EXT,5,
 186,2,"ora",EXT,5,
 187,2,"adda",EXT,5,
 188,2,"cpx",EXT,7,
 189,2,"jsr",EXT,8,
 190,2,"ldx",EXT,6,
 191,2,"stx",EXT,6,

 192,1,"subb",IMM,2,
 193,1,"cmpb",IMM,2,
 194,1,"sbcb",IMM,2,
 195,2,"addd",IMM,4,
 196,1,"andb",IMM,2,
 197,1,"bitb",IMM,2,
 198,1,"ldb",IMM,2,
 200,1,"eorb",IMM,2,
 201,1,"adcb",IMM,2,
 202,1,"orb",IMM,2,
 203,1,"addb",IMM,2,
 204,2,"ldd",IMM,3,
 206,2,"ldu",IMM,3,

 208,1,"subb",DIR,4,
 209,1,"cmpb",DIR,4,
 210,1,"sbcb",DIR,4,
 211,1,"addd",DIR,6,
 212,1,"andb",DIR,4,
 213,1,"bitb",DIR,4,
 214,1,"ldb",DIR,4,
 215,1,"stb",DIR,4,
 216,1,"eorb",DIR,4,
 217,1,"adcb",DIR,4,
 218,1,"orb",DIR,4,
 219,1,"addb",DIR,4,
 220,1,"ldd",DIR,5,
 221,1,"std",DIR,5,
 222,1,"ldu",DIR,5,
 223,1,"stu",DIR,5,

 224,1,"subb",IND,4,
 225,1,"cmpb",IND,4,
 226,1,"sbcb",IND,4,
 227,1,"addd",IND,6,
 228,1,"andb",IND,4,
 229,1,"bitb",IND,4,
 230,1,"ldb",IND,4,
 231,1,"stb",IND,4,
 232,1,"eorb",IND,4,
 233,1,"adcb",IND,4,
 234,1,"orb",IND,4,
 235,1,"addb",IND,4,
 236,1,"ldd",IND,5,
 237,1,"std",IND,5,
 238,1,"ldu",IND,5,
 239,1,"stu",IND,5,

 240,2,"subb",EXT,5,
 241,2,"cmpb",EXT,5,
 242,2,"sbcb",EXT,5,
 243,2,"addd",EXT,7,
 244,2,"andb",EXT,5,
 245,2,"bitb",EXT,5,
 246,2,"ldb",EXT,5,
 247,2,"stb",EXT,5,
 248,2,"eorb",EXT,5,
 249,2,"adcb",EXT,5,
 250,2,"orb",EXT,5,
 251,2,"addb",EXT,5,
 252,2,"ldd",EXT,6,
 253,2,"std",EXT,6,
 254,2,"ldu",EXT,6,
 255,2,"stu",EXT,6,
};

opcodeinfo pg2opcodes[NUMPG2OPS]=
{                       /* page 2 ops 10xx*/
 33,3,"lbrn",LREL,5,
 34,3,"lbhi",LREL,5,
 35,3,"lbls",LREL,5,
 36,3,"lbcc",LREL,5,
 37,3,"lbcs",LREL,5,
 38,3,"lbne",LREL,5,
 39,3,"lbeq",LREL,5,
 40,3,"lbvc",LREL,5,
 41,3,"lbvs",LREL,5,
 42,3,"lbpl",LREL,5,
 43,3,"lbmi",LREL,5,
 44,3,"lbge",LREL,5,
 45,3,"lblt",LREL,5,
 46,3,"lbgt",LREL,5,
 47,3,"lble",LREL,5,
 63,1,"swi2",INH,20, /* chris changed numoperands from 2 to 1 */
 131,3,"cmpd",IMM,5,
 140,3,"cmpy",IMM,5,
 142,3,"ldy",IMM,4,
 147,2,"cmpd",DIR,7,
 156,2,"cmpy",DIR,7,
 158,2,"ldy",DIR,6,
 159,2,"sty",DIR,6,
 163,2,"cmpd",IND,7,
 172,2,"cmpy",IND,7,
 174,2,"ldy",IND,6,
 175,2,"sty",IND,6,
 179,3,"cmpd",EXT,8,
 188,3,"cmpy",EXT,8,
 190,3,"ldy",EXT,7,
 191,3,"sty",EXT,7,
 206,3,"lds",IMM,4,
 222,2,"lds",DIR,6,
 223,2,"sts",DIR,6,
 238,2,"lds",IND,6,
 239,2,"sts",IND,6,
 254,3,"lds",EXT,7,
 255,3,"sts",EXT,7,
};

opcodeinfo pg3opcodes[NUMPG3OPS]=
{                      /* page 3 ops 11xx */
 63,1,"swi3",INH,20,
 131,3,"cmpu",IMM,5,
 140,3,"cmps",IMM,5,
 147,2,"cmpu",DIR,7,
 156,2,"cmps",DIR,7,
 163,2,"cmpu",IND,7,
 172,2,"cmps",IND,7,
 179,3,"cmpu",EXT,8,
 188,3,"cmps",EXT,8,
};
   
opcodeinfo *pgpointers[3]=
{
 pg1opcodes,pg2opcodes,pg3opcodes,
};
